<?php
/**
 * @file
 * Allows the site to send and receive user contacts to and from Text Marketer.
 */

/**
 * Connects to Text Marketer and prepares the data.
 *
 * @return array
 *   Returns an array of matching and non-matching results.
 */
function textmarketer_contacts_prepare_results() {
  $telephone_field_name = variable_get('textmarketer_contacts_telephone_number',
    '');
  $group_name = variable_get('textmarketer_contacts_group_name', '');
  list($user_data, $base_url) = textmarketer_contacts_prepare_data();
  $url = $base_url . 'group/' . $group_name;
  $request = drupal_http_request($url);

  if ($request->code == 200) {
    $data = (isset($request->data)) ? $request->data : '';
    // Convert XML to JSON and process the data.
    $xml = simplexml_load_string($data);
    $json_decode = json_decode(json_encode($xml), TRUE);
    $tel_numbers = (isset($json_decode['group']['number'])) ?
      $json_decode['group']['number'] : '';
    $numbers = array();
    $matching_results = $non_matching = array();

    if (is_array($tel_numbers)) {
      foreach ($tel_numbers as $number) {
        $numbers[] = substr_replace($number, 0, 0, 2);
      }
    }
    else {
      // Convert a string to array when we have one telephone number.
      $numbers = explode(' ', substr_replace((string) $tel_numbers, 0, 0, 2));
    }
    $phone_field = variable_get('textmarketer_contacts_telephone_number',
      '');
    $field_info = field_info_field($phone_field);
    foreach ($field_info['bundles'] as $type => $bundle) {
      $bundle_type = $type;
    }
    foreach ($numbers as $number) {
      $query = db_select('field_data_' . $telephone_field_name, 't');
      $query->fields('t', array('entity_type'));
      $query->addField('t', $telephone_field_name . '_value', 'telephone');
      // Check if we have a matching telephone number.
      if ($bundle_type == 'profile2') {
        $match = $query
          ->condition($telephone_field_name . '_value', db_like($number), 'LIKE')
          ->condition('t.entity_type', $bundle_type, '=')
          ->countQuery()
          ->execute()
          ->fetchField();
        if ($match) {
          // If there is a matching telephone get the rest of the user details.
          $query->Join('profile', 'p', 't.entity_id = p.pid');
          $query->Join('users', 'u', 'p.uid = u.uid');
          $query->fields('u', array('uid', 'name', 'mail', 'created'));
          $query->orderBy('created', 'DESC');
          $matching_results[] = $query->execute()->fetchAll();
        }
      }
      elseif ($bundle_type == 'user') {
        $match = $query
          ->condition($telephone_field_name . '_value', db_like($number), 'LIKE')
          ->condition('t.entity_type', $bundle_type, '=')
          ->countQuery()
          ->execute()
          ->fetchField();

        if ($match) {
          // If there is a matching telephone get the rest of the user details.
          $query->Join('users', 'u', 't.entity_id = u.uid');
          $query->fields('u', array('uid', 'name', 'mail', 'created'));
          $query->orderBy('created', 'DESC');
          $matching_results[] = $query->execute()->fetchAll();
        }
      }
      else {
        $non_matching[] = $number;
      }
    }
    return [$matching_results, $non_matching];
  }
  else {
    drupal_set_message(t('Connection failed, check the !settings_page.',
      array(
        '!settings_page' => l(t('settings page'),
          '/admin/config/services/textmarketer-send-contacts'),
      )), 'error');
  }
}

/**
 * Builds a table of matching user details.
 *
 * @return string
 *   Returns markup and the themed table.
 *
 * @throws \Exception
 *   Returns a notice if no matching numbers found.
 */
function textmarketer_contacts_import_contacts() {
  $output = '<h4>' . t('Contacts matching the site users will be listed in a
  table.') . '</h4>';
  list($matching_results, $non_matching)
    = textmarketer_contacts_prepare_results();

  if ($matching_results) {
    $output .= '<div class="textmarketer-download-csv">' . l(t('Dowload CSV'),
        'admin/config/services/textmarketer-send-contacts/export-contacts') .
      '</div>';
    // Build the table header and rows.
    $header = array(
      'Telephone',
      'User ID',
      'Username',
      'Email address',
      'Created',
    );
    $rows = array();

    drupal_set_message(t('We found @matching @users with matching
      mobile number, and @non_matching @numbers did not match any users.',
      array(
        '@matching' => count($matching_results),
        '@non_matching' => count($non_matching),
        '@users' => format_plural(count($matching_results), 'user', 'users'),
        '@numbers' => format_plural(count($non_matching), 'number', 'numbers'),
      )), 'notice');

    foreach ($matching_results as $user_details) {
      foreach ($user_details as $row) {
        if ($row->uid > 0) {
          $rows[] = array(
            $row->telephone,
            $row->uid,
            l($row->name, 'user/' . $row->uid . '/edit'),
            $row->mail,
            date('d F Y', $row->created),
          );
        }
      }
    }
    $output .= theme('table', array(
      'header' => $header,
      'rows' => $rows,
    ));
    // Add a pager.
    $output .= theme('pager');
  }
  else {
    drupal_set_message(t('There were no matching results.'), 'warning');
  }

  return $output;
}

/**
 * Prepares the CSV file and forces it download.
 */
function textmarketer_contacts_export_contacts() {
  list($matching_results) = textmarketer_contacts_prepare_results();

  foreach ($matching_results as $result) {
    foreach ($result as $user) {
      $user_details[] = $user;
    }
  }

  // Sends response headers to the browser.
  drupal_add_http_header('Content-Type', 'text/csv');
  drupal_add_http_header('Content-Disposition',
    'attachment;filename=csvfile.csv');

  $fp = fopen('php://output', 'w');
  // Build the columns.
  fputcsv($fp, array('Number', 'Username', 'Email'));

  foreach ($user_details as $row) {
    unset($row->uid);
    unset($row->created);
    fputcsv($fp, (array) $row);
  }
  fclose($fp);
  drupal_exit();
}
